// Copyright (c) 2012 Rowley Associates Limited.
//
// This file may be distributed under the terms of the License Agreement
// provided with this software.
//
// THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////////////

#include "LPC407x_8x_177x_8x.h"
#include "system_LPC407x_8x_177x_8x.h"
#include "LPC1000.h"

#define SDRAM_BASE 0xA0000000
#define SDRAM_SIZE (32 * 1024 * 1024)

static void
delay_ms(uint32_t ms)
{
  unsigned long st;
  LPC_TIM0->TCR = 2;
  LPC_TIM0->TCR = 0;
  LPC_TIM0->PR = SystemCoreClock / (LPC_SC->PCLKSEL & LPC_SYSCON_PCLKSEL_PCLKDIV_MASK) / 1000;
  LPC_TIM0->TCR = 1;
  st = LPC_TIM0->TC;
  while ((LPC_TIM0->TC - st) < ms);
}

static uint32_t
calibrate_ring_oscillator()
{
  uint32_t result = 0;
  for (uint32_t i = 0; i < 10; i++)
    {
      uint32_t value;
      LPC_SC->EMCCAL = LPC_SYSCON_EMCCAL_START;
      while (((value = LPC_SC->EMCCAL) & LPC_SYSCON_EMCCAL_DONE) == 0);
      result += (value & LPC_SYSCON_EMCCAL_CALVALUE_MASK);
    }
  return result / 10;
}

static void
adjust_emc_timing(uint32_t cmd_delay, uint32_t fb_delay)
{
  LPC_SC->EMCDLYCTL = ((LPC_SC->EMCDLYCTL & 0x1F1F) * cmd_delay / fb_delay) & 0x1F1F;
}

static int
sdram_ok(void)
{
  volatile uint32_t *ptr32;
  volatile uint16_t *ptr16;
  uint32_t i, j;

  ptr32 = (uint32_t *)SDRAM_BASE;
  ptr16 = (uint16_t *)ptr32;

  /* Fill SDRAM with 16 bit writes */
  for (i = 0; i < SDRAM_SIZE / 0x40000; i++)
    for (j = 0; j < 0x100; j++)
      {
        *ptr16++ = (i + j);
        *ptr16++ = (i + j) + 1;
      }

  /* Verify SDRAM with 32 bit reads */
  ptr32 = (uint32_t *)SDRAM_BASE;
  for (i = 0; i < SDRAM_SIZE / 0x40000; i++)
    for (j = 0; j < 0x100; j++)
      if (*ptr32++ != (((((i + j) + 1) & 0xFFFF) << 16) | ((i + j) & 0xFFFF)))
        return 0;

  return 1;
}

static void
find_and_set_delay(unsigned bit)
{
  int32_t min_delay = -1;
  int32_t max_delay = -1;
  int32_t delay;

  /* Work out best value for delay */
  for (delay = 0; delay < 32; delay++)
    {
      LPC_SC->EMCDLYCTL = (LPC_SC->EMCDLYCTL & ~(0x1F << bit)) | (delay << bit);
      if (sdram_ok()) 
        {
          if (min_delay == -1)
            min_delay = delay;
        } 
      else
        {
          if (min_delay != -1)
            max_delay = delay;
        }
    }
  if (max_delay != -1)
    delay = (min_delay + max_delay) / 2; /* Found max and minimum delay, use average */
  else if (min_delay != -1)
    delay = (min_delay + 0x1F) / 2; /* Only found minimum delay */
  else 
    while (1); /* No delay found */

  /* Setup delay */
  LPC_SC->EMCDLYCTL = (LPC_SC->EMCDLYCTL & ~(0x1F << bit)) | (delay << bit);
}


static void
configure_sdram()
{
  volatile uint32_t i;
  uint32_t cmd_delay, fb_delay;

  LPC_SC->PCONP |= 0x00000800;
  LPC_SC->EMCDLYCTL = 0x00001010;
  LPC_EMC->Control = 0x00000001;
  LPC_EMC->Config = 0x00000000;		
  
  LPC_IOCON->P3_0 = 1; /* D0 */
  LPC_IOCON->P3_1 = 1; /* D1 */
  LPC_IOCON->P3_2 = 1; /* D2 */
  LPC_IOCON->P3_3 = 1; /* D3 */
  LPC_IOCON->P3_4 = 1; /* D4 */
  LPC_IOCON->P3_5 = 1; /* D5 */
  LPC_IOCON->P3_6 = 1; /* D6 */
  LPC_IOCON->P3_7 = 1; /* D7 */
  LPC_IOCON->P3_8 = 1; /* D8 */
  LPC_IOCON->P3_9 = 1; /* D9 */
  LPC_IOCON->P3_10 = 1; /* D10 */
  LPC_IOCON->P3_11 = 1; /* D11 */
  LPC_IOCON->P3_12 = 1; /* D12 */
  LPC_IOCON->P3_13 = 1; /* D13 */
  LPC_IOCON->P3_14 = 1; /* D14 */
  LPC_IOCON->P3_15 = 1; /* D15 */
  LPC_IOCON->P3_16 = 1; /* D16 */
  LPC_IOCON->P3_17 = 1; /* D17 */
  LPC_IOCON->P3_18 = 1; /* D18 */
  LPC_IOCON->P3_19 = 1; /* D19 */
  LPC_IOCON->P3_20 = 1; /* D20 */
  LPC_IOCON->P3_21 = 1; /* D21 */
  LPC_IOCON->P3_22 = 1; /* D22 */
  LPC_IOCON->P3_23 = 1; /* D23 */
  LPC_IOCON->P3_24 = 1; /* D24 */
  LPC_IOCON->P3_25 = 1; /* D25 */
  LPC_IOCON->P3_26 = 1; /* D26 */
  LPC_IOCON->P3_27 = 1; /* D27 */
  LPC_IOCON->P3_28 = 1; /* D28 */
  LPC_IOCON->P3_29 = 1; /* D29 */
  LPC_IOCON->P3_30 = 1; /* D30 */
  LPC_IOCON->P3_31 = 1; /* D31 */

  LPC_IOCON->P4_0 = 1; /* A0 */
  LPC_IOCON->P4_1 = 1; /* A1 */
  LPC_IOCON->P4_2 = 1; /* A2 */
  LPC_IOCON->P4_3 = 1; /* A3 */
  LPC_IOCON->P4_4 = 1; /* A4 */
  LPC_IOCON->P4_5 = 1; /* A5 */
  LPC_IOCON->P4_6 = 1; /* A6 */
  LPC_IOCON->P4_7 = 1; /* A7 */
  LPC_IOCON->P4_8 = 1; /* A8 */
  LPC_IOCON->P4_9 = 1; /* A9 */
  LPC_IOCON->P4_10 = 1; /* A10 */
  LPC_IOCON->P4_11 = 1; /* A11 */
  LPC_IOCON->P4_12 = 1; /* A12 */
  LPC_IOCON->P4_13 = 1; /* A13 */
  LPC_IOCON->P4_14 = 1; /* A14 */
  LPC_IOCON->P4_15 = 1; /* A15 */
  LPC_IOCON->P4_16 = 1; /* A16 */
  LPC_IOCON->P4_17 = 1; /* A17 */
  LPC_IOCON->P4_18 = 1; /* A18 */
  LPC_IOCON->P4_19 = 1; /* A19 */
  LPC_IOCON->P4_20 = 1; /* A20 */
  LPC_IOCON->P4_21 = 1; /* A21 */
  LPC_IOCON->P4_22 = 1; /* A22 */
  LPC_IOCON->P4_23 = 1; /* A23 */

  LPC_IOCON->P4_24 = 1; /* OEN */
  LPC_IOCON->P4_25 = 1; /* WEN */
  LPC_IOCON->P4_26 = 1; /* BLSN[0] */
  LPC_IOCON->P4_27 = 1; /* BLSN[1] */
  LPC_IOCON->P4_28 = 1; /* BLSN[2] */
  LPC_IOCON->P4_29 = 1; /* BLSN[3] */
  LPC_IOCON->P4_30 = 1; /* CSN[0] */
  LPC_IOCON->P4_31 = 1; /* CSN[1] */
  LPC_IOCON->P2_14 = 1; /* CSN[2] */
  LPC_IOCON->P2_15 = 1; /* CSN[3] */
  LPC_IOCON->P2_16 = 1; /* CASN */
  LPC_IOCON->P2_17 = 1; /* RASN */
  LPC_IOCON->P2_18 = 1; /* CLK[0] */
  LPC_IOCON->P2_19 = 1; /* CLK[1] */
  LPC_IOCON->P2_20 = 1; /* DYCSN[0] */
  LPC_IOCON->P2_21 = 1; /* DYCSN[1] */
  LPC_IOCON->P2_22 = 1; /* DYCSN[2] */
  LPC_IOCON->P2_23 = 1; /* DYCSN[3] */
  LPC_IOCON->P2_24 = 1; /* CKE[0] */
  LPC_IOCON->P2_25 = 1; /* CKE[1] */
  LPC_IOCON->P2_26 = 1; /* CKE[2] */
  // LPC_IOCON->P2_27 = 1; /* CKE[3] */
  LPC_IOCON->P2_28 = 1; /* DQM[0] */
  LPC_IOCON->P2_29 = 1; /* DQM[1] */
  LPC_IOCON->P2_30 = 1; /* DQM[2] */
  LPC_IOCON->P2_31 = 1; /* DQM[3] */

  /* Configure SDRAM timings for a 60MHz bus */
  LPC_EMC->DynamicConfig0    = 0x00004480; /* 256MB, 8Mx32, 4 banks, row=12, column=9 */
  LPC_EMC->DynamicRasCas0    = 0x00000202; /* 2 RAS, 2 CAS latency */
  LPC_EMC->DynamicReadConfig = 0x00000001; /* Command delayed strategy, using EMCCLKDELAY */
  LPC_EMC->DynamicRP         = 0x00000001; /* ( n + 1 ) -> 2 clock cycles */
  LPC_EMC->DynamicRAS        = 0x00000003; /* ( n + 1 ) -> 4 clock cycles */
  LPC_EMC->DynamicSREX       = 0x00000005; /* ( n + 1 ) -> 6 clock cycles */
  LPC_EMC->DynamicAPR        = 0x00000002; /* ( n + 1 ) -> 3 clock cycles */
  LPC_EMC->DynamicDAL        = 0x00000003; /* ( n ) -> 3 clock cycles */
  LPC_EMC->DynamicWR         = 0x00000001; /* ( n + 1 ) -> 2 clock cycles */
  LPC_EMC->DynamicRC         = 0x00000004; /* ( n + 1 ) -> 5 clock cycles */
  LPC_EMC->DynamicRFC        = 0x00000004; /* ( n + 1 ) -> 5 clock cycles */
  LPC_EMC->DynamicXSR        = 0x00000005; /* ( n + 1 ) -> 6 clock cycles */
  LPC_EMC->DynamicRRD        = 0x00000001; /* ( n + 1 ) -> 2 clock cycles */
  LPC_EMC->DynamicMRD        = 0x00000001; /* ( n + 1 ) -> 2 clock cycles */
  delay_ms(100);
  LPC_EMC->DynamicControl    = 0x00000183; /* Issue NOP command */
  delay_ms(200);
  LPC_EMC->DynamicControl    = 0x00000103; /* Issue PALL command */
  LPC_EMC->DynamicRefresh    = 0x00000002; /* ( n * 16 ) -> 32 clock cycles */
  for (i = 0; i < 0x80; i++);              /* wait 128 AHB clock cycles */
  LPC_EMC->DynamicRefresh    = 0x0000003A; /* ( n * 16 ) -> 928 clock cycles -> 15.466uS at 60MHz <= 15.625uS ( 64ms / 4096 row ) */
  LPC_EMC->DynamicControl    = 0x00000083; /* Issue MODE command */
  i = *((volatile uint32_t *)(SDRAM_BASE | (0x22 << (2 + 2 + 9)))); /* 4 burst, 2 CAS latency */

  LPC_EMC->DynamicControl    = 0x00000000; /* Issue NORMAL command */
  LPC_EMC->DynamicConfig0    = 0x00084480; /* 256MB, 8Mx32, 4 banks, row=12, column=9 */

  cmd_delay = calibrate_ring_oscillator();  
  find_and_set_delay(LPC_SYSCON_EMCDLYCTL_CMDDLY_BIT);
  find_and_set_delay(LPC_SYSCON_EMCDLYCTL_FBCLKDLY_BIT);
  fb_delay = calibrate_ring_oscillator();
  adjust_emc_timing(cmd_delay, fb_delay);
}

void EMCInit(void);
void EMCInit()
{
  SystemCoreClockUpdate();
  configure_sdram();
}

